package com.m3u.data.worker

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.Icon
import androidx.hilt.work.HiltWorker
import androidx.work.Constraints
import androidx.work.CoroutineWorker
import androidx.work.ForegroundInfo
import androidx.work.NetworkType
import androidx.work.OneTimeWorkRequestBuilder
import androidx.work.OutOfQuotaPolicy
import androidx.work.WorkManager
import androidx.work.WorkerParameters
import androidx.work.workDataOf
import com.m3u.core.architecture.logger.Logger
import com.m3u.core.architecture.logger.Profiles
import com.m3u.core.architecture.logger.install
import com.m3u.core.architecture.logger.post
import com.m3u.data.R
import com.m3u.data.database.model.DataSource
import com.m3u.data.parser.xtream.XtreamInput
import com.m3u.data.repository.PlaylistRepository
import com.m3u.i18n.R.string
import dagger.assisted.Assisted
import dagger.assisted.AssistedInject
import kotlinx.coroutines.CancellationException
import kotlinx.coroutines.Job
import kotlinx.coroutines.coroutineScope
import java.util.concurrent.atomic.AtomicInteger

@HiltWorker
class SubscriptionWorker @AssistedInject constructor(
    @Assisted private val context: Context,
    @Assisted params: WorkerParameters,
    private val playlistRepository: PlaylistRepository,
    private val notificationManager: NotificationManager,
    private val workManager: WorkManager,
    delegate: Logger
) : CoroutineWorker(context, params) {
    private val logger = delegate.install(Profiles.WORKER_SUBSCRIPTION)

    private val dataSource = inputData
        .getString(INPUT_STRING_DATA_SOURCE_VALUE)
        ?.let { DataSource.ofOrNull(it) }

    private val title = inputData.getString(INPUT_STRING_TITLE)
    private val basicUrl = inputData.getString(INPUT_STRING_BASIC_URL)
    private val username = inputData.getString(INPUT_STRING_USERNAME)
    private val password = inputData.getString(INPUT_STRING_PASSWORD)
    private val url = inputData.getString(INPUT_STRING_URL)
    private val epg = inputData.getString(INPUT_STRING_EPG)
    private val notificationId: Int by lazy(LazyThreadSafetyMode.SYNCHRONIZED) {
        ATOMIC_NOTIFICATION_ID.incrementAndGet()
    }

    override suspend fun doWork(): Result = coroutineScope {
        dataSource ?: return@coroutineScope Result.failure()
        createChannel()
        coroutineContext[Job]?.invokeOnCompletion { cause ->
            if (cause is CancellationException) {
                notificationManager.cancel(notificationId)
            }
        }
        when (dataSource) {
            DataSource.M3U -> {
                val title = title ?: return@coroutineScope Result.failure()
                val url = url ?: return@coroutineScope Result.failure()
                if (title.isEmpty()) {
                    val message = context.getString(string.data_error_empty_title)
                    createN10nBuilder()
                        .setContentText(message)
                        .buildThenNotify()
                    Result.failure()
                } else {
                    try {
                        playlistRepository.m3uOrThrow(title, url, epg) { count ->
                            val notification = createN10nBuilder()
                                .setContentText(findProgressContentText(count))
                                .setActions(cancelAction)
                                .setOngoing(true)
                                .build()
                            notificationManager.notify(notificationId, notification)
                            logger.post { "m3u callback" }
                        }

                        logger.post { "m3u suspend resumed" }
                        createN10nBuilder()
                            .setContentText(findCompleteContentText())
                            .buildThenNotify()
                        Result.success()
                    } catch (e: Exception) {
                        createN10nBuilder()
                            .setContentText(e.localizedMessage.orEmpty())
                            .setActions(retryAction)
                            .setColor(Color.RED)
                            .buildThenNotify()
                        e.printStackTrace()
                        Result.failure()
                    }
                }
            }

            DataSource.Xtream -> {
                title ?: return@coroutineScope Result.failure()
                basicUrl ?: return@coroutineScope Result.failure()
                username ?: return@coroutineScope Result.failure()
                password ?: return@coroutineScope Result.failure()
                if (title.isEmpty()) {
                    url ?: return@coroutineScope Result.failure()
                    val message = context.getString(string.data_error_empty_title)
                    createN10nBuilder()
                        .setContentText(message)
                        .buildThenNotify()
                    Result.failure()
                } else {
                    try {
                        val type = url?.let { XtreamInput.decodeFromPlaylistUrlOrNull(it)?.type }
                        playlistRepository.xtreamOrThrow(
                            title, basicUrl, username, password, type
                        ) { count ->
                            val notification = createN10nBuilder()
                                .setContentText(findProgressContentText(count))
                                .setActions(cancelAction)
                                .build()
                            notificationManager.notify(notificationId, notification)
                            logger.post { "xtream callback" }
                        }
                        logger.post { "xtream suspend resumed" }
                        createN10nBuilder()
                            .setContentText(findCompleteContentText())
                            .buildThenNotify()
                        Result.success()
                    } catch (e: Exception) {
                        createN10nBuilder()
                            .setContentText(e.localizedMessage.orEmpty())
                            .setActions(retryAction)
                            .setColor(Color.RED)
                            .buildThenNotify()
                        Result.failure()
                    }
                }
            }

            else -> {
                // do nothing
                Result.failure()
            }
        }
    }

    private fun createChannel() {
        val channel = NotificationChannel(
            CHANNEL_ID, NOTIFICATION_NAME, NotificationManager.IMPORTANCE_DEFAULT
        )
        channel.description = "display subscribe task progress"
        notificationManager.createNotificationChannel(channel)
    }

    private fun Notification.Builder.buildThenNotify() {
        if (isStopped) return
        notificationManager.notify(notificationId, build())
    }

    override suspend fun getForegroundInfo(): ForegroundInfo {
        return ForegroundInfo(notificationId, createN10nBuilder().build())
    }

    private fun createN10nBuilder(): Notification.Builder =
        Notification.Builder(context, CHANNEL_ID)
            .setSmallIcon(R.drawable.round_file_download_24)
            .setContentTitle(title)

    private fun findCancelActionTitle() =
        context.getString(string.data_worker_subscription_action_cancel)

    private fun findRetryActionTitle() =
        context.getString(string.data_worker_subscription_action_retry)

    private fun findCompleteContentText() =
        context.getString(string.data_worker_subscription_content_completed)

    private fun findProgressContentText(count: Int) =
        context.getString(string.data_worker_subscription_content_progress, count)

    private val cancelAction: Notification.Action by lazy {
        Notification.Action.Builder(
            Icon.createWithResource(
                context,
                R.drawable.round_cancel_24
            ),
            findCancelActionTitle(),
            workManager.createCancelPendingIntent(id)
        )
            .build()
    }
    private val retryAction: Notification.Action by lazy {
        Notification.Action.Builder(
            Icon.createWithResource(
                context,
                R.drawable.round_refresh_24
            ),
            findRetryActionTitle(),
            PendingIntent.getForegroundService(
                context,
                1234,
                Intent(),
                PendingIntent.FLAG_IMMUTABLE or PendingIntent.FLAG_UPDATE_CURRENT
            )
        )
            .build()
    }

    companion object {
        private const val CHANNEL_ID = "subscribe_channel"
        private const val NOTIFICATION_NAME = "subscribe task"
        private const val INPUT_STRING_TITLE = "title"
        private const val INPUT_STRING_URL = "url"
        private const val INPUT_STRING_EPG = "epg"
        private const val INPUT_STRING_BASIC_URL = "basic_url"
        private const val INPUT_STRING_USERNAME = "username"
        private const val INPUT_STRING_PASSWORD = "password"
        private const val INPUT_STRING_DATA_SOURCE_VALUE = "data-source"
        const val TAG = "subscription"

        fun m3u(
            workManager: WorkManager,
            title: String,
            url: String,
            epg: String? = null
        ) {
            workManager.cancelAllWorkByTag(url)
            val request = OneTimeWorkRequestBuilder<SubscriptionWorker>()
                .setInputData(
                    workDataOf(
                        INPUT_STRING_TITLE to title,
                        INPUT_STRING_URL to url,
                        INPUT_STRING_EPG to epg,
                        INPUT_STRING_DATA_SOURCE_VALUE to DataSource.M3U.value
                    )
                )
                .addTag(url)
                .addTag(TAG)
                .setExpedited(OutOfQuotaPolicy.RUN_AS_NON_EXPEDITED_WORK_REQUEST)
                .setConstraints(
                    Constraints.Builder()
                        .setRequiredNetworkType(NetworkType.CONNECTED)
                        .build()
                )
                .build()
            workManager.enqueue(request)
        }

        fun xtream(
            workManager: WorkManager,
            title: String,
            url: String,
            basicUrl: String,
            username: String,
            password: String,
        ) {
            workManager.cancelAllWorkByTag(url)
            workManager.cancelAllWorkByTag(basicUrl)
            val request = OneTimeWorkRequestBuilder<SubscriptionWorker>()
                .setInputData(
                    workDataOf(
                        INPUT_STRING_TITLE to title,
                        INPUT_STRING_URL to url,
                        INPUT_STRING_BASIC_URL to basicUrl,
                        INPUT_STRING_USERNAME to username,
                        INPUT_STRING_PASSWORD to password,
                        INPUT_STRING_DATA_SOURCE_VALUE to DataSource.Xtream.value
                    )
                )
                .addTag(url)
                .addTag(basicUrl)
                .apply {
                    val xtreamInput = XtreamInput.decodeFromPlaylistUrlOrNull(url) ?: XtreamInput(
                        basicUrl = basicUrl,
                        username = username,
                        password = password
                    )
                    if (xtreamInput.type == null) {
                        addTag(
                            XtreamInput.encodeToPlaylistUrl(
                                xtreamInput.copy(
                                    type = DataSource.Xtream.TYPE_LIVE
                                )
                            )
                        )
                        addTag(
                            XtreamInput.encodeToPlaylistUrl(
                                xtreamInput.copy(
                                    type = DataSource.Xtream.TYPE_SERIES
                                )
                            )
                        )
                        addTag(
                            XtreamInput.encodeToPlaylistUrl(
                                xtreamInput.copy(
                                    type = DataSource.Xtream.TYPE_VOD
                                )
                            )
                        )
                    }
                }
                .addTag(TAG)
                .setExpedited(OutOfQuotaPolicy.RUN_AS_NON_EXPEDITED_WORK_REQUEST)
                .setConstraints(
                    Constraints.Builder()
                        .setRequiredNetworkType(NetworkType.CONNECTED)
                        .build()
                )
                .build()
            workManager.enqueue(request)
        }

        private val ATOMIC_NOTIFICATION_ID = AtomicInteger()
    }
}
